@@ -26,10 +26,8 @@
 
 using namespace mozilla;
 
-nsresult
-nsWindowMediator::GetDOMWindow(nsIXULWindow* inWindow,
-                               nsCOMPtr<nsPIDOMWindowOuter>& outDOMWindow)
-{
+nsresult nsWindowMediator::GetDOMWindow(
+    nsIXULWindow* inWindow, nsCOMPtr<nsPIDOMWindowOuter>& outDOMWindow) {
   nsCOMPtr<nsIDocShell> docShell;
 
   outDOMWindow = nullptr;
@@ -40,23 +38,22 @@ nsWindowMediator::GetDOMWindow(nsIXULWindow* inWindow,
   return outDOMWindow ? NS_OK : NS_ERROR_FAILURE;
 }
 
-nsWindowMediator::nsWindowMediator() :
-  mEnumeratorList(), mOldestWindow(nullptr), mTopmostWindow(nullptr),
-  mTimeStamp(0), mSortingZOrder(false), mReady(false)
-{
+nsWindowMediator::nsWindowMediator()
+    : mEnumeratorList(),
+      mOldestWindow(nullptr),
+      mTopmostWindow(nullptr),
+      mTimeStamp(0),
+      mSortingZOrder(false),
+      mReady(false) {}
+
+nsWindowMediator::~nsWindowMediator() {
+  while (mOldestWindow) UnregisterWindow(mOldestWindow);
 }
 
-nsWindowMediator::~nsWindowMediator()
-{
-  while (mOldestWindow)
-    UnregisterWindow(mOldestWindow);
-}
-
-nsresult nsWindowMediator::Init()
-{
+nsresult nsWindowMediator::Init() {
   nsresult rv;
   nsCOMPtr<nsIObserverService> obsSvc =
-    do_GetService("@mozilla.org/observer-service;1", &rv);
+      do_GetService("@mozilla.org/observer-service;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = obsSvc->AddObserver(this, "xpcom-shutdown", true);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -65,8 +62,7 @@ nsresult nsWindowMediator::Init()
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWindowMediator::RegisterWindow(nsIXULWindow* inWindow)
-{
+NS_IMETHODIMP nsWindowMediator::RegisterWindow(nsIXULWindow* inWindow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_STATE(mReady);
 
@@ -94,19 +90,15 @@ NS_IMETHODIMP nsWindowMediator::RegisterWindow(nsIXULWindow* inWindow)
 }
 
 NS_IMETHODIMP
-nsWindowMediator::UnregisterWindow(nsIXULWindow* inWindow)
-{
+nsWindowMediator::UnregisterWindow(nsIXULWindow* inWindow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_STATE(mReady);
-  nsWindowInfo *info = GetInfoFor(inWindow);
-  if (info)
-    return UnregisterWindow(info);
+  nsWindowInfo* info = GetInfoFor(inWindow);
+  if (info) return UnregisterWindow(info);
   return NS_ERROR_INVALID_ARG;
 }
 
-nsresult
-nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
-{
+nsresult nsWindowMediator::UnregisterWindow(nsWindowInfo* inInfo) {
   // Inform the iterators
   uint32_t index = 0;
   while (index < mEnumeratorList.Length()) {
@@ -121,48 +113,35 @@ nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
   }
 
   // Remove from the lists and free up
-  if (inInfo == mOldestWindow)
-    mOldestWindow = inInfo->mYounger;
-  if (inInfo == mTopmostWindow)
-    mTopmostWindow = inInfo->mLower;
+  if (inInfo == mOldestWindow) mOldestWindow = inInfo->mYounger;
+  if (inInfo == mTopmostWindow) mTopmostWindow = inInfo->mLower;
   inInfo->Unlink(true, true);
-  if (inInfo == mOldestWindow)
-    mOldestWindow = nullptr;
-  if (inInfo == mTopmostWindow)
-    mTopmostWindow = nullptr;
+  if (inInfo == mOldestWindow) mOldestWindow = nullptr;
+  if (inInfo == mTopmostWindow) mTopmostWindow = nullptr;
   delete inInfo;
 
   return NS_OK;
 }
 
-nsWindowInfo*
-nsWindowMediator::GetInfoFor(nsIXULWindow *aWindow)
-{
-  nsWindowInfo *info,
-               *listEnd;
+nsWindowInfo* nsWindowMediator::GetInfoFor(nsIXULWindow* aWindow) {
+  nsWindowInfo *info, *listEnd;
 
-  if (!aWindow)
-    return nullptr;
+  if (!aWindow) return nullptr;
 
   info = mOldestWindow;
   listEnd = nullptr;
   while (info != listEnd) {
-    if (info->mWindow.get() == aWindow)
-      return info;
+    if (info->mWindow.get() == aWindow) return info;
     info = info->mYounger;
     listEnd = mOldestWindow;
   }
   return nullptr;
 }
 
-nsWindowInfo*
-nsWindowMediator::GetInfoFor(nsIWidget *aWindow)
-{
-  nsWindowInfo *info,
-               *listEnd;
+nsWindowInfo* nsWindowMediator::GetInfoFor(nsIWidget* aWindow) {
+  nsWindowInfo *info, *listEnd;
 
-  if (!aWindow)
-    return nullptr;
+  if (!aWindow) return nullptr;
 
   info = mOldestWindow;
   listEnd = nullptr;
@@ -170,10 +149,8 @@ nsWindowMediator::GetInfoFor(nsIWidget *aWindow)
   nsCOMPtr<nsIWidget> scanWidget;
   while (info != listEnd) {
     nsCOMPtr<nsIBaseWindow> base(do_QueryInterface(info->mWindow));
-    if (base)
-      base->GetMainWidget(getter_AddRefs(scanWidget));
-    if (aWindow == scanWidget.get())
-      return info;
+    if (base) base->GetMainWidget(getter_AddRefs(scanWidget));
+    if (aWindow == scanWidget.get()) return info;
     info = info->mYounger;
     listEnd = mOldestWindow;
   }
@@ -181,53 +158,35 @@ nsWindowMediator::GetInfoFor(nsIWidget *aWindow)
 }
 
 NS_IMETHODIMP
-nsWindowMediator::GetEnumerator(const char16_t* inType, nsISimpleEnumerator** outEnumerator)
-{
+nsWindowMediator::GetEnumerator(const char16_t* inType,
+                                nsISimpleEnumerator** outEnumerator) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(outEnumerator);
   NS_ENSURE_STATE(mReady);
 
-  RefPtr<nsAppShellWindowEnumerator> enumerator = new nsASDOMWindowEarlyToLateEnumerator(inType, *this);
+  RefPtr<nsAppShellWindowEnumerator> enumerator =
+      new nsASDOMWindowEarlyToLateEnumerator(inType, *this);
   enumerator.forget(outEnumerator);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWindowMediator::GetXULWindowEnumerator(const char16_t* inType, nsISimpleEnumerator** outEnumerator)
-{
+nsWindowMediator::GetXULWindowEnumerator(const char16_t* inType,
+                                         nsISimpleEnumerator** outEnumerator) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(outEnumerator);
   NS_ENSURE_STATE(mReady);
 
-  RefPtr<nsAppShellWindowEnumerator> enumerator = new nsASXULWindowEarlyToLateEnumerator(inType, *this);
+  RefPtr<nsAppShellWindowEnumerator> enumerator =
+      new nsASXULWindowEarlyToLateEnumerator(inType, *this);
   enumerator.forget(outEnumerator);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWindowMediator::GetZOrderDOMWindowEnumerator(
-            const char16_t *aWindowType, bool aFrontToBack,
-            nsISimpleEnumerator **_retval)
-{
-  MOZ_RELEASE_ASSERT(NS_IsMainThread());
-  NS_ENSURE_ARG_POINTER(_retval);
-  NS_ENSURE_STATE(mReady);
-
-  RefPtr<nsAppShellWindowEnumerator> enumerator;
-  if (aFrontToBack)
-    enumerator = new nsASDOMWindowFrontToBackEnumerator(aWindowType, *this);
-  else
-    enumerator = new nsASDOMWindowBackToFrontEnumerator(aWindowType, *this);
-
-  enumerator.forget(_retval);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWindowMediator::GetZOrderXULWindowEnumerator(
-            const char16_t *aWindowType, bool aFrontToBack,
-            nsISimpleEnumerator **_retval)
-{
+nsWindowMediator::GetZOrderXULWindowEnumerator(const char16_t* aWindowType,
+                                               bool aFrontToBack,
+                                               nsISimpleEnumerator** _retval) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(_retval);
   NS_ENSURE_STATE(mReady);
@@ -242,29 +201,25 @@ nsWindowMediator::GetZOrderXULWindowEnumerator(
   return NS_OK;
 }
 
-int32_t
-nsWindowMediator::AddEnumerator(nsAppShellWindowEnumerator * inEnumerator)
-{
+int32_t nsWindowMediator::AddEnumerator(
+    nsAppShellWindowEnumerator* inEnumerator) {
   return mEnumeratorList.AppendElement(inEnumerator) != nullptr;
 }
 
-int32_t
-nsWindowMediator::RemoveEnumerator(nsAppShellWindowEnumerator * inEnumerator)
-{
+int32_t nsWindowMediator::RemoveEnumerator(
+    nsAppShellWindowEnumerator* inEnumerator) {
   return mEnumeratorList.RemoveElement(inEnumerator);
 }
 
-// Returns the window of type inType ( if null return any window type ) which has the most recent
-// time stamp
+// Returns the window of type inType ( if null return any window type ) which
+// has the most recent time stamp
 NS_IMETHODIMP
 nsWindowMediator::GetMostRecentWindow(const char16_t* inType,
-                                      mozIDOMWindowProxy** outWindow)
-{
+                                      mozIDOMWindowProxy** outWindow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(outWindow);
   *outWindow = nullptr;
-  if (!mReady)
-    return NS_OK;
+  if (!mReady) return NS_OK;
 
   // Find the most window with the highest time stamp that matches
   // the requested type
@@ -282,13 +237,28 @@ nsWindowMediator::GetMostRecentWindow(const char16_t* inType,
 }
 
 NS_IMETHODIMP
-nsWindowMediator::GetMostRecentNonPBWindow(const char16_t* aType, mozIDOMWindowProxy** aWindow)
-{
+nsWindowMediator::GetMostRecentBrowserWindow(mozIDOMWindowProxy** outWindow) {
+  nsresult rv = GetMostRecentWindow(u"navigator:browser", outWindow);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+#ifdef MOZ_WIDGET_ANDROID
+  if (!*outWindow) {
+    rv = GetMostRecentWindow(u"navigator:geckoview", outWindow);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+#endif
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowMediator::GetMostRecentNonPBWindow(const char16_t* aType,
+                                           mozIDOMWindowProxy** aWindow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(aWindow);
   *aWindow = nullptr;
 
-  nsWindowInfo *info = MostRecentWindowInfo(aType, true);
+  nsWindowInfo* info = MostRecentWindowInfo(aType, true);
   nsCOMPtr<nsPIDOMWindowOuter> domWindow;
   if (info && info->mWindow) {
     GetDOMWindow(info->mWindow, domWindow);
@@ -302,13 +272,11 @@ nsWindowMediator::GetMostRecentNonPBWindow(const char16_t* aType, mozIDOMWindowP
   return NS_OK;
 }
 
-nsWindowInfo*
-nsWindowMediator::MostRecentWindowInfo(const char16_t* inType,
-                                       bool aSkipPrivateBrowsingOrClosed)
-{
-  int32_t       lastTimeStamp = -1;
-  nsAutoString  typeString(inType);
-  bool          allWindows = !inType || typeString.IsEmpty();
+nsWindowInfo* nsWindowMediator::MostRecentWindowInfo(
+    const char16_t* inType, bool aSkipPrivateBrowsingOrClosed) {
+  int32_t lastTimeStamp = -1;
+  nsAutoString typeString(inType);
+  bool allWindows = !inType || typeString.IsEmpty();
 
   // Find the most recent window with the highest time stamp that matches
   // the requested type and has the correct browsing mode.
@@ -350,42 +318,37 @@ nsWindowMediator::MostRecentWindowInfo(const char16_t* inType,
 
 NS_IMETHODIMP
 nsWindowMediator::GetOuterWindowWithId(uint64_t aWindowID,
-                                       mozIDOMWindowProxy** aWindow)
-{
-  RefPtr<nsGlobalWindowOuter> window = nsGlobalWindowOuter::GetOuterWindowWithId(aWindowID);
-  nsCOMPtr<nsPIDOMWindowOuter> outer = window ? window->AsOuter() : nullptr;
-  outer.forget(aWindow);
+                                       mozIDOMWindowProxy** aWindow) {
+  RefPtr<nsGlobalWindowOuter> window =
+      nsGlobalWindowOuter::GetOuterWindowWithId(aWindowID);
+  window.forget(aWindow);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsWindowMediator::GetCurrentInnerWindowWithId(uint64_t aWindowID,
-                                              mozIDOMWindow** aWindow)
-{
-  RefPtr<nsGlobalWindowInner> window = nsGlobalWindowInner::GetInnerWindowWithId(aWindowID);
+                                              mozIDOMWindow** aWindow) {
+  RefPtr<nsGlobalWindowInner> window =
+      nsGlobalWindowInner::GetInnerWindowWithId(aWindowID);
 
   // not found
-  if (!window)
-    return NS_OK;
+  if (!window) return NS_OK;
 
-  nsCOMPtr<nsPIDOMWindowInner> inner = window->AsInner();
-  nsCOMPtr<nsPIDOMWindowOuter> outer = inner->GetOuterWindow();
+  nsCOMPtr<nsPIDOMWindowOuter> outer = window->GetOuterWindow();
   NS_ENSURE_TRUE(outer, NS_ERROR_UNEXPECTED);
 
   // outer is already using another inner, so it's same as not found
-  if (outer->GetCurrentInnerWindow() != inner)
-    return NS_OK;
+  if (outer->GetCurrentInnerWindow() != window) return NS_OK;
 
-  inner.forget(aWindow);
+  window.forget(aWindow);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWindowMediator::UpdateWindowTimeStamp(nsIXULWindow* inWindow)
-{
+nsWindowMediator::UpdateWindowTimeStamp(nsIXULWindow* inWindow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_STATE(mReady);
-  nsWindowInfo *info = GetInfoFor(inWindow);
+  nsWindowInfo* info = GetInfoFor(inWindow);
   if (info) {
     // increment the window's time stamp
     info->mTimeStamp = ++mTimeStamp;
@@ -401,37 +364,32 @@ nsWindowMediator::UpdateWindowTimeStamp(nsIXULWindow* inWindow)
    calculate a change as seldom as possible.
 */
 NS_IMETHODIMP
-nsWindowMediator::CalculateZPosition(
-                nsIXULWindow   *inWindow,
-                uint32_t        inPosition,
-                nsIWidget      *inBelow,
-                uint32_t       *outPosition,
-                nsIWidget     **outBelow,
-                bool           *outAltered)
-{
+nsWindowMediator::CalculateZPosition(nsIXULWindow* inWindow,
+                                     uint32_t inPosition, nsIWidget* inBelow,
+                                     uint32_t* outPosition,
+                                     nsIWidget** outBelow, bool* outAltered) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(outBelow);
   NS_ENSURE_STATE(mReady);
 
   *outBelow = nullptr;
 
-  if (!inWindow || !outPosition || !outAltered)
-    return NS_ERROR_NULL_POINTER;
+  if (!inWindow || !outPosition || !outAltered) return NS_ERROR_NULL_POINTER;
 
   if (inPosition != nsIWindowMediator::zLevelTop &&
       inPosition != nsIWindowMediator::zLevelBottom &&
       inPosition != nsIWindowMediator::zLevelBelow)
     return NS_ERROR_INVALID_ARG;
 
-  nsWindowInfo *info = mTopmostWindow;
-  nsIXULWindow *belowWindow = nullptr;
-  bool          found = false;
-  nsresult      result = NS_OK;
+  nsWindowInfo* info = mTopmostWindow;
+  nsIXULWindow* belowWindow = nullptr;
+  bool found = false;
+  nsresult result = NS_OK;
 
   *outPosition = inPosition;
   *outAltered = false;
 
-  if (mSortingZOrder) { // don't fight SortZOrder()
+  if (mSortingZOrder) {  // don't fight SortZOrder()
     *outBelow = inBelow;
     NS_IF_ADDREF(*outBelow);
     return NS_OK;
@@ -463,8 +421,7 @@ nsWindowMediator::CalculateZPosition(
     if (mTopmostWindow && mTopmostWindow->mZLevel > inZ) {
       // asked for topmost, can't have it. locate highest allowed position.
       do {
-        if (info->mZLevel <= inZ)
-          break;
+        if (info->mZLevel <= inZ) break;
         info = info->mLower;
       } while (info != mTopmostWindow);
 
@@ -477,8 +434,7 @@ nsWindowMediator::CalculateZPosition(
       // asked for bottommost, can't have it. locate lowest allowed position.
       do {
         info = info->mHigher;
-        if (info->mZLevel >= inZ)
-          break;
+        if (info->mZLevel >= inZ) break;
       } while (info != mTopmostWindow);
 
       *outPosition = nsIWindowMediator::zLevelBelow;
@@ -496,8 +452,7 @@ nsWindowMediator::CalculateZPosition(
         // might be OK. is lower window, if any, lower?
         if (info->mLower != info && info->mLower->mZLevel > inZ) {
           do {
-            if (info->mZLevel <= inZ)
-              break;
+            if (info->mZLevel <= inZ) break;
             info = info->mLower;
           } while (info != mTopmostWindow);
 
@@ -508,8 +463,7 @@ nsWindowMediator::CalculateZPosition(
         // nope. look for a higher window to be behind.
         do {
           info = info->mHigher;
-          if (info->mZLevel >= inZ)
-            break;
+          if (info->mZLevel >= inZ) break;
         } while (info != mTopmostWindow);
 
         if (info->mZLevel >= inZ)
@@ -517,7 +471,7 @@ nsWindowMediator::CalculateZPosition(
         else
           *outPosition = nsIWindowMediator::zLevelTop;
         *outAltered = true;
-      } // else they're equal, so it's OK
+      }  // else they're equal, so it's OK
     }
   }
 
@@ -533,14 +487,10 @@ nsWindowMediator::CalculateZPosition(
 }
 
 NS_IMETHODIMP
-nsWindowMediator::SetZPosition(
-                nsIXULWindow *inWindow,
-                uint32_t      inPosition,
-                nsIXULWindow *inBelow)
-{
+nsWindowMediator::SetZPosition(nsIXULWindow* inWindow, uint32_t inPosition,
+                               nsIXULWindow* inBelow) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
-  nsWindowInfo *inInfo,
-               *belowInfo;
+  nsWindowInfo *inInfo, *belowInfo;
 
   if ((inPosition != nsIWindowMediator::zLevelTop &&
        inPosition != nsIWindowMediator::zLevelBottom &&
@@ -549,7 +499,7 @@ nsWindowMediator::SetZPosition(
     return NS_ERROR_INVALID_ARG;
   }
 
-  if (mSortingZOrder) // don't fight SortZOrder()
+  if (mSortingZOrder)  // don't fight SortZOrder()
     return NS_OK;
 
   NS_ENSURE_STATE(mReady);
@@ -559,15 +509,14 @@ nsWindowMediator::SetZPosition(
      This is because the former is guaranteed complete, while
      now may be this window's first exposure to the latter. */
   inInfo = GetInfoFor(inWindow);
-  if (!inInfo)
-    return NS_ERROR_INVALID_ARG;
+  if (!inInfo) return NS_ERROR_INVALID_ARG;
 
   // locate inBelow, place inWindow behind it
   if (inPosition == nsIWindowMediator::zLevelBelow) {
     belowInfo = GetInfoFor(inBelow);
     // it had better also be in the z-order list
-    if (belowInfo &&
-        belowInfo->mYounger != belowInfo && belowInfo->mLower == belowInfo) {
+    if (belowInfo && belowInfo->mYounger != belowInfo &&
+        belowInfo->mLower == belowInfo) {
       belowInfo = nullptr;
     }
     if (!belowInfo) {
@@ -585,19 +534,17 @@ nsWindowMediator::SetZPosition(
     inInfo->Unlink(false, true);
     inInfo->InsertAfter(nullptr, belowInfo);
   }
-  if (inPosition == nsIWindowMediator::zLevelTop)
-    mTopmostWindow = inInfo;
+  if (inPosition == nsIWindowMediator::zLevelTop) mTopmostWindow = inInfo;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWindowMediator::GetZLevel(nsIXULWindow *aWindow, uint32_t *_retval)
-{
+nsWindowMediator::GetZLevel(nsIXULWindow* aWindow, uint32_t* _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
   *_retval = nsIXULWindow::normalZ;
   // This can fail during window destruction.
-  nsWindowInfo *info = GetInfoFor(aWindow);
+  nsWindowInfo* info = GetInfoFor(aWindow);
   if (info) {
     *_retval = info->mZLevel;
   }
@@ -605,15 +552,13 @@ nsWindowMediator::GetZLevel(nsIXULWindow *aWindow, uint32_t *_retval)
 }
 
 NS_IMETHODIMP
-nsWindowMediator::SetZLevel(nsIXULWindow *aWindow, uint32_t aZLevel)
-{
+nsWindowMediator::SetZLevel(nsIXULWindow* aWindow, uint32_t aZLevel) {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   NS_ENSURE_STATE(mReady);
 
-  nsWindowInfo *info = GetInfoFor(aWindow);
+  nsWindowInfo* info = GetInfoFor(aWindow);
   NS_ASSERTION(info, "setting z level of unregistered window");
-  if (!info)
-    return NS_ERROR_FAILURE;
+  if (!info) return NS_ERROR_FAILURE;
 
   if (info->mZLevel != aZLevel) {
     bool lowered = info->mZLevel > aZLevel;
@@ -638,17 +583,15 @@ nsWindowMediator::SetZLevel(nsIXULWindow *aWindow, uint32_t aZLevel)
    on slow systems windows could visibly bubble around the window that
    was moved.
 */
-void
-nsWindowMediator::SortZOrderFrontToBack()
-{
-  nsWindowInfo *scan,   // scans list looking for problems
-               *search, // searches for correct placement for scan window
-               *prev,   // previous search element
-               *lowest; // bottom-most window in list
-  bool         finished;
+void nsWindowMediator::SortZOrderFrontToBack() {
+  nsWindowInfo *scan,  // scans list looking for problems
+      *search,         // searches for correct placement for scan window
+      *prev,           // previous search element
+      *lowest;         // bottom-most window in list
+  bool finished;
 
-  if (!mTopmostWindow) // early during program execution there's no z list yet
-    return;            // there's also only one window, so this is not dangerous
+  if (!mTopmostWindow)  // early during program execution there's no z list yet
+    return;  // there's also only one window, so this is not dangerous
 
   mSortingZOrder = true;
 
@@ -660,7 +603,7 @@ nsWindowMediator::SortZOrderFrontToBack()
     scan = mTopmostWindow;
     while (scan != lowest) {
       uint32_t scanZ = scan->mZLevel;
-      if (scanZ < scan->mLower->mZLevel) { // out of order
+      if (scanZ < scan->mLower->mZLevel) {  // out of order
         search = scan->mLower;
         do {
           prev = search;
@@ -668,8 +611,7 @@ nsWindowMediator::SortZOrderFrontToBack()
         } while (prev != lowest && scanZ < search->mZLevel);
 
         // reposition |scan| within the list
-        if (scan == mTopmostWindow)
-          mTopmostWindow = scan->mLower;
+        if (scan == mTopmostWindow) mTopmostWindow = scan->mLower;
         scan->Unlink(false, true);
         scan->InsertAfter(nullptr, prev);
 
@@ -678,11 +620,9 @@ nsWindowMediator::SortZOrderFrontToBack()
         nsCOMPtr<nsIWidget> scanWidget;
         nsCOMPtr<nsIWidget> prevWidget;
         base = do_QueryInterface(scan->mWindow);
-        if (base)
-          base->GetMainWidget(getter_AddRefs(scanWidget));
+        if (base) base->GetMainWidget(getter_AddRefs(scanWidget));
         base = do_QueryInterface(prev->mWindow);
-        if (base)
-          base->GetMainWidget(getter_AddRefs(prevWidget));
+        if (base) base->GetMainWidget(getter_AddRefs(prevWidget));
         if (scanWidget)
           scanWidget->PlaceBehind(eZPlacementBelow, prevWidget, false);
 
@@ -697,16 +637,14 @@ nsWindowMediator::SortZOrderFrontToBack()
 }
 
 // see comment for SortZOrderFrontToBack
-void
-nsWindowMediator::SortZOrderBackToFront()
-{
-  nsWindowInfo *scan,   // scans list looking for problems
-               *search, // searches for correct placement for scan window
-               *lowest; // bottom-most window in list
-  bool         finished;
+void nsWindowMediator::SortZOrderBackToFront() {
+  nsWindowInfo *scan,  // scans list looking for problems
+      *search,         // searches for correct placement for scan window
+      *lowest;         // bottom-most window in list
+  bool finished;
 
-  if (!mTopmostWindow) // early during program execution there's no z list yet
-    return;            // there's also only one window, so this is not dangerous
+  if (!mTopmostWindow)  // early during program execution there's no z list yet
+    return;  // there's also only one window, so this is not dangerous
 
   mSortingZOrder = true;
 
@@ -718,7 +656,7 @@ nsWindowMediator::SortZOrderBackToFront()
     scan = lowest;
     while (scan != mTopmostWindow) {
       uint32_t scanZ = scan->mZLevel;
-      if (scanZ > scan->mHigher->mZLevel) { // out of order
+      if (scanZ > scan->mHigher->mZLevel) {  // out of order
         search = scan;
         do {
           search = search->mHigher;
@@ -729,20 +667,17 @@ nsWindowMediator::SortZOrderBackToFront()
           scan->Unlink(false, true);
           scan->InsertAfter(nullptr, search);
         }
-        if (search == lowest)
-          mTopmostWindow = scan;
+        if (search == lowest) mTopmostWindow = scan;
 
         // fix actual window order
         nsCOMPtr<nsIBaseWindow> base;
         nsCOMPtr<nsIWidget> scanWidget;
         nsCOMPtr<nsIWidget> searchWidget;
         base = do_QueryInterface(scan->mWindow);
-        if (base)
-          base->GetMainWidget(getter_AddRefs(scanWidget));
+        if (base) base->GetMainWidget(getter_AddRefs(scanWidget));
         if (mTopmostWindow != scan) {
           base = do_QueryInterface(search->mWindow);
-          if (base)
-            base->GetMainWidget(getter_AddRefs(searchWidget));
+          if (base) base->GetMainWidget(getter_AddRefs(searchWidget));
         }
         if (scanWidget)
           scanWidget->PlaceBehind(eZPlacementBelow, searchWidget, false);
@@ -756,15 +691,11 @@ nsWindowMediator::SortZOrderBackToFront()
   mSortingZOrder = false;
 }
 
-NS_IMPL_ISUPPORTS(nsWindowMediator,
-  nsIWindowMediator_44,
-  nsIWindowMediator,
-  nsIObserver,
-  nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(nsWindowMediator, nsIWindowMediator, nsIObserver,
+                  nsISupportsWeakReference)
 
 NS_IMETHODIMP
-nsWindowMediator::AddListener(nsIWindowMediatorListener* aListener)
-{
+nsWindowMediator::AddListener(nsIWindowMediatorListener* aListener) {
   NS_ENSURE_ARG_POINTER(aListener);
 
   mListeners.AppendElement(aListener);
@@ -773,8 +704,7 @@ nsWindowMediator::AddListener(nsIWindowMediatorListener* aListener)
 }
 
 NS_IMETHODIMP
-nsWindowMediator::RemoveListener(nsIWindowMediatorListener* aListener)
-{
+nsWindowMediator::RemoveListener(nsIWindowMediatorListener* aListener) {
   NS_ENSURE_ARG_POINTER(aListener);
 
   mListeners.RemoveElement(aListener);
@@ -783,14 +713,11 @@ nsWindowMediator::RemoveListener(nsIWindowMediatorListener* aListener)
 }
 
 NS_IMETHODIMP
-nsWindowMediator::Observe(nsISupports* aSubject,
-                          const char* aTopic,
-                          const char16_t* aData)
-{
+nsWindowMediator::Observe(nsISupports* aSubject, const char* aTopic,
+                          const char16_t* aData) {
   if (!strcmp(aTopic, "xpcom-shutdown") && mReady) {
     MOZ_RELEASE_ASSERT(NS_IsMainThread());
-    while (mOldestWindow)
-      UnregisterWindow(mOldestWindow);
+    while (mOldestWindow) UnregisterWindow(mOldestWindow);
     mReady = false;
   }
   return NS_OK;
